unit switch;

interface
 const
  SW_NONE=0;
  SW_EXIT=1;
  SW_EXITS=2;
  SW_OPENDOOR=3;
  SW_SHUTDOOR=4;
  SW_SHUTTRAP=5;
  SW_DOOR=6;
  SW_DOOR5=7;
  SW_PRESS=8;
  SW_TELE=9;
  SW_SECRET=10;
  SW_LIFTUP=11;
  SW_LIFTDOWN=12;
  SW_TRAP=13;
  SW_LIFT=14;

  SW_PL_PRESS=1;
  SW_MN_PRESS=2;
  SW_PL_NEAR=4;
  SW_MN_NEAR=8;
  SW_KEY_R=16;
  SW_KEY_G=32;
  SW_KEY_B=64;

 procedure addSwitch(_type, x, y, a, b, flag:integer);
 procedure pressSwitch(x, y, w, h, flags, user_id:integer);
 procedure resetAll;
 procedure update;

implementation
 uses vars, monster_struct;
 const
  MAXSW=100;
 var
  sw_type, sw_x, sw_y: array [0..MAXSW] of integer;
  sw_a, sw_b, sw_d: array [0..MAXSW] of integer;
  sw_flag: array [0..MAXSW] of integer;
  sw_count:integer;

  max_secrets, secrets:integer;

  sw_time:integer;
  cht,chto,chf,f_ch:integer;

 procedure resetAll;
  var
   i:integer;
  begin
   for i:=0 to MAXSW do
    begin
     sw_type[i]:=SW_NONE;
     sw_x[i]:=0;
     sw_y[i]:=0;
     sw_a[i]:=0;
     sw_b[i]:=0;
     sw_d[i]:=0;
     sw_flag[i]:=0;
    end;
   sw_count:=0;
   max_secrets:=0;
   secrets:=0;
  end;

 procedure addSwitch(_type, x, y, a, b, flag:integer);
  begin
   sw_type[sw_count]:=_type;
   sw_x[sw_count]:=x;
   sw_y[sw_count]:=y;
   sw_a[sw_count]:=a;
   sw_b[sw_count]:=b;
   sw_d[sw_count]:=0;
   sw_flag[sw_count]:=flag;
   sw_count:=sw_count+1;

   if _type=SW_SECRET then
    max_secrets:=max_secrets+1;
  end;

 procedure door(x, y:integer);
  var
   ex, i:integer;
  begin
   if (x>=getMapW) or (y>=getMapH) then
    exit;
   if getMap(x, y)<>cht then
    exit;

   ex:=x+1;

   while (x>0) and (getMap(x-1, y)=cht) do
    x:=x-1;

   while (ex<getMapW) and (getMap(ex, y)=cht) do
    ex:=ex+1;

   for i:=0 to ex-x-1 do
    setMap(chto, x+i, y);

   if f_ch>0 then
    for i:=0 to ex-x-1 do
     setMapFore(chf, x+i, y);

   for i:=x to ex-1 do
    begin
     door(i, y-1);
     door(i, y+1);
    end;
  end;

 procedure opendoor(x, y:integer);
  var
   tmp:integer;
  begin
   tmp:=getMapFore(x, y);
   cht:=MAPTYPE_CDOOR; chto:=MAPTYPE_ODOOR; chf:=0; f_ch:=1;
   door(x, y);
   setMapFore(tmp, x, y);
  end;

 procedure closedoor(x, y:integer);
  begin
   chf:=getMapFore(x, y);
   cht:=MAPTYPE_ODOOR; chto:=MAPTYPE_CDOOR; f_ch:=1;
   door(x, y);
  end;

 procedure liftup(x, y:integer);
  begin
   cht:=MAPTYPE_LIFTD; chto:=MAPTYPE_LIFTU; chf:=0; f_ch:=0;
   door(x, y);
  end;

 procedure liftdown(x, y:integer);
  begin
   cht:=MAPTYPE_LIFTU; chto:=MAPTYPE_LIFTD; chf:=0; f_ch:=0;
   door(x, y);
  end;

 procedure s_door(x, y:integer);
  var
   tmp:integer;
  begin
   tmp:=getMap(x, y);
   if tmp=MAPTYPE_CDOOR then
    opendoor(x, y);
   else
   if tmp=MAPTYPE_ODOOR then
    closedoor(x, y);
  end;

 procedure update;
  var
   _type, a, b, d, i, time:integer;
  begin
   time:=GetRelativeTimeMs-sw_time;
   for i:=0 to MAXSW do
    begin
     _type:=sw_type[i];
     a:=sw_a[i];
     b:=sw_b[i];
     d:=sw_d[i];

     if _type=SW_DOOR5 then
      begin
       if d>0 then
        begin
         d:=d-time;
         if d=0 then
          d:=-1;
        end;
       if d<0 then
        begin
         d:=0;
         closedoor(a, b);
        end;
       sw_d[i]:=d;
      end;

    end;
   sw_time:=GetRelativeTimeMs;
  end;

 procedure useSwitch(i, flags, user_id:integer);
  var
   _type, a, b, tmp:integer;
  begin
   _type:=sw_type[i];
   a:=sw_a[i];
   b:=sw_b[i];
   debug('use switch #'+i+' type: '+_type);

   if _type=SW_EXIT then
    exit_lvl:=1;
   else
   if _type=SW_EXITS then
    exit_lvl:=2;
   else
   if _type=SW_OPENDOOR then
    opendoor(a, b);
   else
   if _type=SW_SHUTDOOR then
    closedoor(a, b);
   else
   if _type=SW_DOOR then
    s_door(a, b);
   else
   if _type=SW_DOOR5 then
    begin
     s_door(a, b);
     sw_d[i]:=4950;
    end;
   else
   if _type=SW_SECRET then
    begin
     if sw_d[i]=0 then
      begin
       secrets:=secrets+1;
       sw_d[i]:=1;
      end;
    end;
   else
   if _type=SW_PRESS then
    pressSwitch(a*TILE_SIZE+TILE_SIZE/2,
                b*TILE_SIZE+TILE_SIZE/2+TILE_SIZE-1,
                8,
                16,
                (flags and $70) or $80,
                user_id);
   else
   if _type=SW_TELE then
    begin
     debug('SW_TELE a:'+a+' b:'+b+' i:'+user_id);
     monster_struct.setX(a*TILE_SIZE+TILE_SIZE/4-monster_struct.getW(user_id)/2, user_id);
     monster_struct.setY(b*TILE_SIZE+TILE_SIZE/4+TILE_SIZE-2-monster_struct.getH(user_id), user_id);
    end;
   else
   if _type=SW_LIFTUP then
    liftup(a, b);
   else
   if _type=SW_LIFTDOWN then
    liftdown(a, b);
   else
   if _type=SW_LIFT then
    begin
     tmp:=getMap(a, b);
     if tmp=MAPTYPE_LIFTD then
      liftup(a, b);
     else
     if tmp=MAPTYPE_LIFTU then
      liftdown(a, b);
    end;
   else
  end;

 procedure pressSwitch(x, y, w, h, flags, user_id:integer);
  const
   KEYS_MASK=$70;
   ACT_MASK=$0F;
  var
   i:integer;
  begin
   for i:=0 to MAXSW do
    if sw_type[i]<>SW_NONE then
     begin
      if (sw_x[i]+TILE_SIZE-1>=x) and (sw_x[i]<=x+w) and (sw_y[i]+TILE_SIZE-1>=y) and (sw_y[i]<=y+h) then
       if ((flags and ACT_MASK) and (sw_flag[i] and ACT_MASK))>0 then
       /* Проверка на наличие подходящих ключей
       if ((flags and KEYS_MASK) and (sw_flag[i] and KEYS_MASK))=(sw_flag[i] and KEYS_MASK) then*/
        useSwitch(i, flags, user_id);
     end;
  end;

end.
